@@ -24,16 +24,16 @@ DEALINGS IN THE SOFTWARE.
 
 from __future__ import annotations
 
-from typing import Any, TYPE_CHECKING, List, Optional, Type, Union
+from typing import Any, TYPE_CHECKING, List, Optional, Union
 
-
-from ..enums import AppCommandOptionType, AppCommandType
-from ..errors import DiscordException
+from ..enums import AppCommandOptionType, AppCommandType, Locale
+from ..errors import DiscordException, HTTPException, _flatten_error_dict
 
 __all__ = (
     'AppCommandError',
     'CommandInvokeError',
     'TransformerError',
+    'TranslationError',
     'CheckFailure',
     'CommandAlreadyRegistered',
     'CommandSignatureMismatch',
@@ -46,14 +46,18 @@ __all__ = (
     'BotMissingPermissions',
     'CommandOnCooldown',
     'MissingApplicationID',
+    'CommandSyncFailure',
 )
 
 if TYPE_CHECKING:
     from .commands import Command, Group, ContextMenu
     from .transformers import Transformer
+    from .translator import TranslationContext, locale_str
     from ..types.snowflake import Snowflake, SnowflakeList
     from .checks import Cooldown
 
+    CommandTypes = Union[Command[Any, ..., Any], Group, ContextMenu]
+
 APP_ID_NOT_FOUND = (
     'Client does not have an application_id set. Either the function was called before on_ready '
     'was called or application_id was not passed to the Client constructor.'
@@ -121,28 +125,60 @@ class TransformerError(AppCommandError):
         The value that failed to convert.
     type: :class:`~discord.AppCommandOptionType`
         The type of argument that failed to convert.
-    transformer: Type[:class:`Transformer`]
+    transformer: :class:`Transformer`
         The transformer that failed the conversion.
     """
 
-    def __init__(self, value: Any, opt_type: AppCommandOptionType, transformer: Type[Transformer]):
+    def __init__(self, value: Any, opt_type: AppCommandOptionType, transformer: Transformer):
         self.value: Any = value
         self.type: AppCommandOptionType = opt_type
-        self.transformer: Type[Transformer] = transformer
+        self.transformer: Transformer = transformer
 
-        try:
-            result_type = transformer.transform.__annotations__['return']
-        except KeyError:
-            name = transformer.__name__
-            if name.endswith('Transformer'):
-                result_type = name[:-11]
-            else:
-                result_type = name
+        super().__init__(f'Failed to convert {value} to {transformer._error_display_name!s}')
+
+
+class TranslationError(AppCommandError):
+    """An exception raised when the library fails to translate a string.
+
+    This inherits from :exc:`~discord.app_commands.AppCommandError`.
+
+    If an exception occurs while calling :meth:`Translator.translate` that does
+    not subclass this then the exception is wrapped into this exception.
+    The original exception can be retrieved using the ``__cause__`` attribute.
+    Otherwise it will be propagated as-is.
+
+    .. versionadded:: 2.0
+
+    Attributes
+    -----------
+    string: Optional[Union[:class:`str`, :class:`locale_str`]]
+        The string that caused the error, if any.
+    locale: Optional[:class:`~discord.Locale`]
+        The locale that caused the error, if any.
+    context: :class:`~discord.app_commands.TranslationContext`
+        The context of the translation that triggered the error.
+    """
+
+    def __init__(
+        self,
+        *msg: str,
+        string: Optional[Union[str, locale_str]] = None,
+        locale: Optional[Locale] = None,
+        context: TranslationContext,
+    ) -> None:
+        self.string: Optional[Union[str, locale_str]] = string
+        self.locale: Optional[Locale] = locale
+        self.context: TranslationContext = context
+
+        if msg:
+            super().__init__(*msg)
         else:
-            if isinstance(result_type, type):
-                result_type = result_type.__name__
+            ctx = context.location.name.replace('_', ' ')
+            fmt = f'Failed to translate {self.string!r} in a {ctx}'
+            if self.locale is not None:
+                fmt = f'{fmt} in the {self.locale.value} locale'
 
-        super().__init__(f'Failed to convert {value} to {result_type!s}')
+            super().__init__(fmt)
 
 
 class CheckFailure(AppCommandError):
@@ -410,3 +446,58 @@ class MissingApplicationID(AppCommandError):
 
     def __init__(self, message: Optional[str] = None):
         super().__init__(message or APP_ID_NOT_FOUND)
+
+
+def _get_command_error(index: str, inner: Any, commands: List[CommandTypes], messages: List[str]) -> None:
+    # Top level errors are:
+    # <number>: { <key>: <error> }
+    # The dicts could be nested, e.g.
+    # <number>: { <key>: { <second>: <error> } }
+    # Luckily, this is already handled by the flatten_error_dict utility
+    if not index.isdigit():
+        errors = _flatten_error_dict(inner, index)
+        messages.extend(f'In {k}: {v}' for k, v in errors.items())
+        return
+
+    idx = int(index)
+    try:
+        command = commands[idx]
+    except IndexError:
+        errors = _flatten_error_dict(inner, index)
+        messages.extend(f'In {k}: {v}' for k, v in errors.items())
+        return
+
+    callback = getattr(command, 'callback', None)
+    class_name = command.__class__.__name__
+    if callback:
+        messages.append(f'In {class_name} {command.qualified_name!r} defined in {callback.__qualname__!r}')
+    else:
+        messages.append(f'In {class_name} {command.qualified_name!r} defined in module {command.module!r}')
+
+    errors = _flatten_error_dict(inner)
+    messages.extend(f'  {k}: {v}' for k, v in errors.items())
+
+
+class CommandSyncFailure(AppCommandError, HTTPException):
+    """An exception raised when :meth:`CommandTree.sync` failed.
+
+    This provides syncing failures in a slightly more readable format.
+
+    This inherits from :exc:`~discord.app_commands.AppCommandError`
+    and :exc:`~discord.HTTPException`.
+
+    .. versionadded:: 2.0
+    """
+
+    def __init__(self, child: HTTPException, commands: List[CommandTypes]) -> None:
+        # Consume the child exception and make it seem as if we are that exception
+        self.__dict__.update(child.__dict__)
+
+        messages = [f'Failed to upload commands to Discord (HTTP status {self.status}, error code {self.code})']
+
+        if self._errors:
+            for index, inner in self._errors.items():
+                _get_command_error(index, inner, commands, messages)
+
+        # Equivalent to super().__init__(...) but skips other constructors
+        self.args = ('\n'.join(messages),)
