@@ -46,9 +46,11 @@ from typing import (
 
 from .errors import AppCommandError, TransformerError
 from .models import AppCommandChannel, AppCommandThread, Choice
+from .translator import TranslationContextLocation, locale_str, Translator, TranslationContext
 from ..channel import StageChannel, VoiceChannel, TextChannel, CategoryChannel
+from ..abc import GuildChannel
 from ..threads import Thread
-from ..enums import Enum as InternalEnum, AppCommandOptionType, ChannelType
+from ..enums import Enum as InternalEnum, AppCommandOptionType, ChannelType, Locale
 from ..utils import MISSING, maybe_coroutine
 from ..user import User
 from ..role import Role
@@ -68,34 +70,15 @@ NoneType = type(None)
 
 if TYPE_CHECKING:
     from ..interactions import Interaction
+    from .commands import Parameter
 
 
 @dataclass
 class CommandParameter:
-    """Represents a application command parameter.
-
-    Attributes
-    -----------
-    name: :class:`str`
-        The name of the parameter.
-    description: :class:`str`
-        The description of the parameter
-    required: :class:`bool`
-        Whether the parameter is required
-    choices: List[:class:`~discord.app_commands.Choice`]
-        A list of choices this parameter takes
-    type: :class:`~discord.AppCommandOptionType`
-        The underlying type of this parameter.
-    channel_types: List[:class:`~discord.ChannelType`]
-        The channel types that are allowed for this parameter.
-    min_value: Optional[Union[:class:`int`, :class:`float`]]
-        The minimum supported value for this parameter.
-    max_value: Optional[Union[:class:`int`, :class:`float`]]
-        The maximum supported value for this parameter.
-    """
-
+    # The name of the parameter is *always* the parameter name in the code
+    # Therefore, it can't be Union[str, locale_str]
     name: str = MISSING
-    description: str = MISSING
+    description: Union[str, locale_str] = MISSING
     required: bool = MISSING
     default: Any = MISSING
     choices: List[Choice[Union[str, int, float]]] = MISSING
@@ -104,14 +87,49 @@ class CommandParameter:
     min_value: Optional[Union[int, float]] = None
     max_value: Optional[Union[int, float]] = None
     autocomplete: Optional[Callable[..., Coroutine[Any, Any, Any]]] = None
-    _rename: str = MISSING
+    _rename: Union[str, locale_str] = MISSING
     _annotation: Any = MISSING
 
+    async def get_translated_payload(self, translator: Translator, data: Parameter) -> Dict[str, Any]:
+        base = self.to_dict()
+
+        rename = self._rename
+        description = self.description
+        needs_name_translations = isinstance(rename, locale_str)
+        needs_description_translations = isinstance(description, locale_str)
+        name_localizations: Dict[str, str] = {}
+        description_localizations: Dict[str, str] = {}
+
+        # Prevent creating these objects in a heavy loop
+        name_context = TranslationContext(location=TranslationContextLocation.parameter_name, data=data)
+        description_context = TranslationContext(location=TranslationContextLocation.parameter_description, data=data)
+        for locale in Locale:
+            if needs_name_translations:
+                translation = await translator._checked_translate(rename, locale, name_context)
+                if translation is not None:
+                    name_localizations[locale.value] = translation
+
+            if needs_description_translations:
+                translation = await translator._checked_translate(description, locale, description_context)
+                if translation is not None:
+                    description_localizations[locale.value] = translation
+
+        if self.choices:
+            base['choices'] = [await choice.get_translated_payload(translator) for choice in self.choices]
+
+        if name_localizations:
+            base['name_localizations'] = name_localizations
+
+        if description_localizations:
+            base['description_localizations'] = description_localizations
+
+        return base
+
     def to_dict(self) -> Dict[str, Any]:
         base = {
             'type': self.type.value,
             'name': self.display_name,
-            'description': self.description,
+            'description': str(self.description),
             'required': self.required,
         }
 
@@ -121,13 +139,31 @@ class CommandParameter:
             base['channel_types'] = [t.value for t in self.channel_types]
         if self.autocomplete:
             base['autocomplete'] = True
+
+        min_key, max_key = (
+            ('min_value', 'max_value') if self.type is not AppCommandOptionType.string else ('min_length', 'max_length')
+        )
         if self.min_value is not None:
-            base['min_value'] = self.min_value
+            base[min_key] = self.min_value
         if self.max_value is not None:
-            base['max_value'] = self.max_value
+            base[max_key] = self.max_value
 
         return base
 
+    def _convert_to_locale_strings(self) -> None:
+        if self._rename is MISSING:
+            self._rename = locale_str(self.name)
+        elif isinstance(self._rename, str):
+            self._rename = locale_str(self._rename)
+
+        if isinstance(self.description, str):
+            self.description = locale_str(self.description)
+
+        if self.choices:
+            for choice in self.choices:
+                if choice._locale_name is None:
+                    choice._locale_name = locale_str(choice.name)
+
     def is_choice_annotation(self) -> bool:
         return getattr(self._annotation, '__discord_app_commands_is_choice__', False)
 
@@ -153,7 +189,7 @@ class CommandParameter:
     @property
     def display_name(self) -> str:
         """:class:`str`: The name of the parameter as it should be displayed to the user."""
-        return self.name if self._rename is MISSING else self._rename
+        return self.name if self._rename is MISSING else str(self._rename)
 
 
 class Transformer:
@@ -161,7 +197,7 @@ class Transformer:
     to map into a :class:`~discord.AppCommandOptionType` and transform the raw value into one
     from this type.
 
-    This class is customisable through the overriding of :func:`classmethod` in the class
+    This class is customisable through the overriding of methods and properties in the class
     and by using it as the second type parameter of the :class:`~discord.app_commands.Transform`
     class. For example, to convert a string into a custom pair type:
 
@@ -172,8 +208,7 @@ class Transformer:
             y: int
 
         class PointTransformer(app_commands.Transformer):
-            @classmethod
-            async def transform(cls, interaction: discord.Interaction, value: str) -> Point:
+            async def transform(self, interaction: discord.Interaction, value: str) -> Point:
                 (x, _, y) = value.partition(',')
                 return Point(x=int(x.strip()), y=int(y.strip()))
 
@@ -184,56 +219,93 @@ class Transformer:
         ):
             await interaction.response.send_message(str(point))
 
+    If a class is passed instead of an instance to the second type parameter, then it is
+    constructed with no arguments passed to the ``__init__`` method.
+
     .. versionadded:: 2.0
     """
 
     __discord_app_commands_transformer__: ClassVar[bool] = True
     __discord_app_commands_is_choice__: ClassVar[bool] = False
 
-    @classmethod
-    def type(cls) -> AppCommandOptionType:
+    # This is needed to pass typing's type checks.
+    # e.g. Optional[MyTransformer]
+    def __call__(self) -> None:
+        pass
+
+    def __or__(self, rhs: Any) -> Any:
+        return Union[self, rhs]  # type: ignore
+
+    @property
+    def type(self) -> AppCommandOptionType:
         """:class:`~discord.AppCommandOptionType`: The option type associated with this transformer.
 
-        This must be a :obj:`classmethod`.
+        This must be a :obj:`property`.
 
         Defaults to :attr:`~discord.AppCommandOptionType.string`.
         """
         return AppCommandOptionType.string
 
-    @classmethod
-    def channel_types(cls) -> List[ChannelType]:
+    @property
+    def channel_types(self) -> List[ChannelType]:
         """List[:class:`~discord.ChannelType`]: A list of channel types that are allowed to this parameter.
 
         Only valid if the :meth:`type` returns :attr:`~discord.AppCommandOptionType.channel`.
 
+        This must be a :obj:`property`.
+
         Defaults to an empty list.
         """
         return []
 
-    @classmethod
-    def min_value(cls) -> Optional[Union[int, float]]:
+    @property
+    def min_value(self) -> Optional[Union[int, float]]:
         """Optional[:class:`int`]: The minimum supported value for this parameter.
 
-        Only valid if the :meth:`type` returns :attr:`~discord.AppCommandOptionType.number` or
-        :attr:`~discord.AppCommandOptionType.integer`.
+        Only valid if the :meth:`type` returns :attr:`~discord.AppCommandOptionType.number`
+        :attr:`~discord.AppCommandOptionType.integer`, or :attr:`~discord.AppCommandOptionType.string`.
+
+        This must be a :obj:`property`.
 
         Defaults to ``None``.
         """
         return None
 
-    @classmethod
-    def max_value(cls) -> Optional[Union[int, float]]:
+    @property
+    def max_value(self) -> Optional[Union[int, float]]:
         """Optional[:class:`int`]: The maximum supported value for this parameter.
 
-        Only valid if the :meth:`type` returns :attr:`~discord.AppCommandOptionType.number` or
-        :attr:`~discord.AppCommandOptionType.integer`.
+        Only valid if the :meth:`type` returns :attr:`~discord.AppCommandOptionType.number`
+        :attr:`~discord.AppCommandOptionType.integer`, or :attr:`~discord.AppCommandOptionType.string`.
+
+        This must be a :obj:`property`.
 
         Defaults to ``None``.
         """
         return None
 
-    @classmethod
-    async def transform(cls, interaction: Interaction, value: Any) -> Any:
+    @property
+    def choices(self) -> Optional[List[Choice[Union[int, float, str]]]]:
+        """Optional[List[:class:`~discord.app_commands.Choice`]]: A list of choices that are allowed to this parameter.
+
+        Only valid if the :meth:`type` returns :attr:`~discord.AppCommandOptionType.number`
+        :attr:`~discord.AppCommandOptionType.integer`, or :attr:`~discord.AppCommandOptionType.string`.
+
+        This must be a :obj:`property`.
+
+        Defaults to ``None``.
+        """
+        return None
+
+    @property
+    def _error_display_name(self) -> str:
+        name = self.__class__.__name__
+        if name.endswith('Transformer'):
+            return name[:-11]
+        else:
+            return name
+
+    async def transform(self, interaction: Interaction, value: Any) -> Any:
         """|maybecoro|
 
         Transforms the converted option value into another value.
@@ -252,9 +324,8 @@ class Transformer:
         """
         raise NotImplementedError('Derived classes need to implement this.')
 
-    @classmethod
     async def autocomplete(
-        cls, interaction: Interaction, value: Union[int, float, str]
+        self, interaction: Interaction, value: Union[int, float, str]
     ) -> List[Choice[Union[int, float, str]]]:
         """|coro|
 
@@ -282,119 +353,158 @@ class Transformer:
         raise NotImplementedError('Derived classes can implement this.')
 
 
-class _TransformMetadata:
-    __discord_app_commands_transform__: ClassVar[bool] = True
-    __slots__ = ('metadata',)
+class IdentityTransformer(Transformer):
+    def __init__(self, type: AppCommandOptionType) -> None:
+        self._type = type
 
-    def __init__(self, metadata: Type[Transformer]):
-        self.metadata: Type[Transformer] = metadata
+    @property
+    def type(self) -> AppCommandOptionType:
+        return self._type
 
-    # This is needed to pass typing's type checks.
-    # e.g. Optional[Transform[discord.Member, MyTransformer]]
-    def __call__(self) -> None:
-        pass
+    async def transform(self, interaction: Interaction, value: Any) -> Any:
+        return value
 
 
-async def _identity_transform(cls, interaction: Interaction, value: Any) -> Any:
-    return value
+class RangeTransformer(IdentityTransformer):
+    def __init__(
+        self,
+        opt_type: AppCommandOptionType,
+        *,
+        min: Optional[Union[int, float]] = None,
+        max: Optional[Union[int, float]] = None,
+    ) -> None:
+        if min and max and min > max:
+            raise TypeError('minimum cannot be larger than maximum')
+
+        self._min: Optional[Union[int, float]] = min
+        self._max: Optional[Union[int, float]] = max
+        super().__init__(opt_type)
+
+    @property
+    def min_value(self) -> Optional[Union[int, float]]:
+        return self._min
+
+    @property
+    def max_value(self) -> Optional[Union[int, float]]:
+        return self._max
 
 
-def _make_range_transformer(
-    opt_type: AppCommandOptionType,
-    *,
-    min: Optional[Union[int, float]] = None,
-    max: Optional[Union[int, float]] = None,
-) -> Type[Transformer]:
-    ns = {
-        'type': classmethod(lambda _: opt_type),
-        'min_value': classmethod(lambda _: min),
-        'max_value': classmethod(lambda _: max),
-        'transform': classmethod(_identity_transform),
-    }
-    return type('RangeTransformer', (Transformer,), ns)
+class LiteralTransformer(IdentityTransformer):
+    def __init__(self, values: Tuple[Any, ...]) -> None:
+        first = type(values[0])
+        if first is int:
+            opt_type = AppCommandOptionType.integer
+        elif first is float:
+            opt_type = AppCommandOptionType.number
+        elif first is str:
+            opt_type = AppCommandOptionType.string
+        else:
+            raise TypeError(f'expected int, str, or float values not {first!r}')
+
+        self._choices = [Choice(name=str(v), value=v) for v in values]
+        super().__init__(opt_type)
+
+    @property
+    def choices(self):
+        return self._choices
 
 
-def _make_literal_transformer(values: Tuple[Any, ...]) -> Type[Transformer]:
-    first = type(values[0])
-    if first is int:
-        opt_type = AppCommandOptionType.integer
-    elif first is float:
-        opt_type = AppCommandOptionType.number
-    elif first is str:
-        opt_type = AppCommandOptionType.string
-    else:
-        raise TypeError(f'expected int, str, or float values not {first!r}')
+class ChoiceTransformer(IdentityTransformer):
+    __discord_app_commands_is_choice__: ClassVar[bool] = True
 
-    ns = {
-        'type': classmethod(lambda _: opt_type),
-        'transform': classmethod(_identity_transform),
-        '__discord_app_commands_transformer_choices__': [Choice(name=str(v), value=v) for v in values],
-    }
-    return type('LiteralTransformer', (Transformer,), ns)
+    def __init__(self, inner_type: Any) -> None:
+        if inner_type is int:
+            opt_type = AppCommandOptionType.integer
+        elif inner_type is float:
+            opt_type = AppCommandOptionType.number
+        elif inner_type is str:
+            opt_type = AppCommandOptionType.string
+        else:
+            raise TypeError(f'expected int, str, or float values not {inner_type!r}')
+
+        super().__init__(opt_type)
 
 
-def _make_choice_transformer(inner_type: Any) -> Type[Transformer]:
-    if inner_type is int:
-        opt_type = AppCommandOptionType.integer
-    elif inner_type is float:
-        opt_type = AppCommandOptionType.number
-    elif inner_type is str:
-        opt_type = AppCommandOptionType.string
-    else:
-        raise TypeError(f'expected int, str, or float values not {inner_type!r}')
+class EnumValueTransformer(Transformer):
+    def __init__(self, enum: Any) -> None:
+        super().__init__()
 
-    ns = {
-        'type': classmethod(lambda _: opt_type),
-        'transform': classmethod(_identity_transform),
-        '__discord_app_commands_is_choice__': True,
-    }
-    return type('ChoiceTransformer', (Transformer,), ns)
+        values = list(enum)
+        if len(values) < 2:
+            raise TypeError(f'enum.Enum requires at least two values.')
+
+        first = type(values[0].value)
+        if first is int:
+            opt_type = AppCommandOptionType.integer
+        elif first is float:
+            opt_type = AppCommandOptionType.number
+        elif first is str:
+            opt_type = AppCommandOptionType.string
+        else:
+            raise TypeError(f'expected int, str, or float values not {first!r}')
+
+        self._type: AppCommandOptionType = opt_type
+        self._enum: Any = enum
+        self._choices = [Choice(name=v.name, value=v.value) for v in values]
+
+    @property
+    def _error_display_name(self) -> str:
+        return self._enum.__name__
+
+    @property
+    def type(self) -> AppCommandOptionType:
+        return self._type
+
+    @property
+    def choices(self):
+        return self._choices
+
+    async def transform(self, interaction: Interaction, value: Any) -> Any:
+        return self._enum(value)
 
 
-def _make_enum_transformer(enum) -> Type[Transformer]:
-    values = list(enum)
-    if len(values) < 2:
-        raise TypeError(f'enum.Enum requires at least two values.')
+class EnumNameTransformer(Transformer):
+    def __init__(self, enum: Any) -> None:
+        super().__init__()
 
-    first = type(values[0].value)
-    if first is int:
-        opt_type = AppCommandOptionType.integer
-    elif first is float:
-        opt_type = AppCommandOptionType.number
-    elif first is str:
-        opt_type = AppCommandOptionType.string
-    else:
-        raise TypeError(f'expected int, str, or float values not {first!r}')
+        values = list(enum)
+        if len(values) < 2:
+            raise TypeError(f'enum.Enum requires at least two values.')
 
-    async def transform(cls, interaction: Interaction, value: Any) -> Any:
-        return enum(value)
+        self._enum: Any = enum
+        self._choices = [Choice(name=v.name, value=v.name) for v in values]
 
-    ns = {
-        'type': classmethod(lambda _: opt_type),
-        'transform': classmethod(transform),
-        '__discord_app_commands_transformer_enum__': enum,
-        '__discord_app_commands_transformer_choices__': [Choice(name=v.name, value=v.value) for v in values],
-    }
+    @property
+    def _error_display_name(self) -> str:
+        return self._enum.__name__
 
-    return type(f'{enum.__name__}EnumTransformer', (Transformer,), ns)
+    @property
+    def type(self) -> AppCommandOptionType:
+        return AppCommandOptionType.string
+
+    @property
+    def choices(self):
+        return self._choices
+
+    async def transform(self, interaction: Interaction, value: Any) -> Any:
+        return self._enum[value]
 
 
-def _make_complex_enum_transformer(enum) -> Type[Transformer]:
-    values = list(enum)
-    if len(values) < 2:
-        raise TypeError(f'enum.Enum requires at least two values.')
+class InlineTransformer(Transformer):
+    def __init__(self, annotation: Any) -> None:
+        super().__init__()
+        self.annotation: Any = annotation
 
-    async def transform(cls, interaction: Interaction, value: Any) -> Any:
-        return enum[value]
+    @property
+    def _error_display_name(self) -> str:
+        return self.annotation.__name__
 
-    ns = {
-        'type': classmethod(lambda _: AppCommandOptionType.string),
-        'transform': classmethod(transform),
-        '__discord_app_commands_transformer_enum__': enum,
-        '__discord_app_commands_transformer_choices__': [Choice(name=v.name, value=v.name) for v in values],
-    }
+    @property
+    def type(self) -> AppCommandOptionType:
+        return AppCommandOptionType.string
 
-    return type(f'{enum.__name__}ComplexEnumTransformer', (Transformer,), ns)
+    async def transform(self, interaction: Interaction, value: Any) -> Any:
+        return await self.annotation.transform(interaction, value)
 
 
 if TYPE_CHECKING:
@@ -425,15 +535,18 @@ else:
 
             _, transformer = items
 
-            is_valid = inspect.isclass(transformer) and issubclass(transformer, Transformer)
-            if not is_valid:
-                raise TypeError(f'second argument of Transform must be a Transformer class not {transformer!r}')
+            if inspect.isclass(transformer):
+                if not issubclass(transformer, Transformer):
+                    raise TypeError(f'second argument of Transform must be a Transformer class not {transformer!r}')
+                transformer = transformer()
+            elif not isinstance(transformer, Transformer):
+                raise TypeError(f'second argument of Transform must be a Transformer not {transformer.__class__!r}')
 
-            return _TransformMetadata(transformer)
+            return transformer
 
     class Range:
-        """A type annotation that can be applied to a parameter to require a numeric type
-        to fit within the range provided.
+        """A type annotation that can be applied to a parameter to require a numeric or string
+        type to fit within the range provided.
 
         During type checking time this is equivalent to :obj:`typing.Annotated` so type checkers understand
         the intent of the code.
@@ -479,91 +592,90 @@ else:
                 opt_type = AppCommandOptionType.integer
             elif obj_type is float:
                 opt_type = AppCommandOptionType.number
+            elif obj_type is str:
+                opt_type = AppCommandOptionType.string
             else:
-                raise TypeError(f'expected int or float as range type, received {obj_type!r} instead')
+                raise TypeError(f'expected int, float, or str as range type, received {obj_type!r} instead')
 
-            transformer = _make_range_transformer(
+            if obj_type in (str, int):
+                cast = int
+            else:
+                cast = float
+
+            transformer = RangeTransformer(
                 opt_type,
-                min=obj_type(min) if min is not None else None,
-                max=obj_type(max) if max is not None else None,
+                min=cast(min) if min is not None else None,
+                max=cast(max) if max is not None else None,
             )
-            return _TransformMetadata(transformer)
-
-
-def passthrough_transformer(opt_type: AppCommandOptionType) -> Type[Transformer]:
-    class _Generated(Transformer):
-        @classmethod
-        def type(cls) -> AppCommandOptionType:
-            return opt_type
-
-        @classmethod
-        async def transform(cls, interaction: Interaction, value: Any) -> Any:
-            return value
-
-    return _Generated
+            return transformer
 
 
 class MemberTransformer(Transformer):
-    @classmethod
-    def type(cls) -> AppCommandOptionType:
+    @property
+    def type(self) -> AppCommandOptionType:
         return AppCommandOptionType.user
 
-    @classmethod
-    async def transform(cls, interaction: Interaction, value: Any) -> Member:
+    async def transform(self, interaction: Interaction, value: Any) -> Member:
         if not isinstance(value, Member):
-            raise TransformerError(value, cls.type(), cls)
+            raise TransformerError(value, self.type, self)
         return value
 
 
-def channel_transformer(*channel_types: Type[Any], raw: Optional[bool] = False) -> Type[Transformer]:
-    if raw:
+class BaseChannelTransformer(Transformer):
+    def __init__(self, *channel_types: Type[Any]) -> None:
+        super().__init__()
+        if len(channel_types) == 1:
+            display_name = channel_types[0].__name__
+            types = CHANNEL_TO_TYPES[channel_types[0]]
+        else:
+            display_name = '{}, and {}'.format(', '.join(t.__name__ for t in channel_types[:-1]), channel_types[-1].__name__)
+            types = []
 
-        async def transform(cls, interaction: Interaction, value: Any):
-            if not isinstance(value, channel_types):
-                raise TransformerError(value, AppCommandOptionType.channel, cls)
+            for t in channel_types:
+                try:
+                    types.extend(CHANNEL_TO_TYPES[t])
+                except KeyError:
+                    raise TypeError(f'Union type of channels must be entirely made up of channels') from None
+
+        self._types: Tuple[Type[Any]] = channel_types
+        self._channel_types: List[ChannelType] = types
+        self._display_name = display_name
+
+    @property
+    def _error_display_name(self) -> str:
+        return self._display_name
+
+    @property
+    def type(self) -> AppCommandOptionType:
+        return AppCommandOptionType.channel
+
+    @property
+    def channel_types(self) -> List[ChannelType]:
+        return self._channel_types
+
+    async def transform(self, interaction: Interaction, value: Any):
+        resolved = value.resolve()
+        if resolved is None or not isinstance(resolved, self._types):
+            raise TransformerError(value, AppCommandOptionType.channel, self)
+        return resolved
+
+
+class RawChannelTransformer(BaseChannelTransformer):
+    async def transform(self, interaction: Interaction, value: Any):
+        if not isinstance(value, self._types):
+            raise TransformerError(value, AppCommandOptionType.channel, self)
+        return value
+
+
+class UnionChannelTransformer(BaseChannelTransformer):
+    async def transform(self, interaction: Interaction, value: Any):
+        if isinstance(value, self._types):
             return value
 
-    elif raw is False:
-
-        async def transform(cls, interaction: Interaction, value: Any):
-            resolved = value.resolve()
-            if resolved is None or not isinstance(resolved, channel_types):
-                raise TransformerError(value, AppCommandOptionType.channel, cls)
-            return resolved
-
-    else:
-
-        async def transform(cls, interaction: Interaction, value: Any):
-            if isinstance(value, channel_types):
-                return value
-
-            resolved = value.resolve()
-            if resolved is None or not isinstance(resolved, channel_types):
-                raise TransformerError(value, AppCommandOptionType.channel, cls)
-            return resolved
-
-    if len(channel_types) == 1:
-        name = channel_types[0].__name__
-        types = CHANNEL_TO_TYPES[channel_types[0]]
-    else:
-        name = 'MultiChannel'
-        types = []
-
-        for t in channel_types:
-            try:
-                types.extend(CHANNEL_TO_TYPES[t])
-            except KeyError:
-                raise TypeError(f'Union type of channels must be entirely made up of channels') from None
-
-    return type(
-        f'{name}Transformer',
-        (Transformer,),
-        {
-            'type': classmethod(lambda cls: AppCommandOptionType.channel),
-            'transform': classmethod(transform),
-            'channel_types': classmethod(lambda cls: types),
-        },
-    )
+        resolved = value.resolve()
+        if resolved is None or not isinstance(resolved, self._types):
+            raise TransformerError(value, AppCommandOptionType.channel, self)
+        return resolved
 
 
 CHANNEL_TO_TYPES: Dict[Any, List[ChannelType]] = {
@@ -574,6 +686,13 @@ CHANNEL_TO_TYPES: Dict[Any, List[ChannelType]] = {
         ChannelType.news,
         ChannelType.category,
     ],
+    GuildChannel: [
+        ChannelType.stage_voice,
+        ChannelType.voice,
+        ChannelType.text,
+        ChannelType.news,
+        ChannelType.category,
+    ],
     AppCommandThread: [ChannelType.news_thread, ChannelType.private_thread, ChannelType.public_thread],
     Thread: [ChannelType.news_thread, ChannelType.private_thread, ChannelType.public_thread],
     StageChannel: [ChannelType.stage_voice],
@@ -582,22 +701,23 @@ CHANNEL_TO_TYPES: Dict[Any, List[ChannelType]] = {
     CategoryChannel: [ChannelType.category],
 }
 
-BUILT_IN_TRANSFORMERS: Dict[Any, Type[Transformer]] = {
-    str: passthrough_transformer(AppCommandOptionType.string),
-    int: passthrough_transformer(AppCommandOptionType.integer),
-    float: passthrough_transformer(AppCommandOptionType.number),
-    bool: passthrough_transformer(AppCommandOptionType.boolean),
-    User: passthrough_transformer(AppCommandOptionType.user),
-    Member: MemberTransformer,
-    Role: passthrough_transformer(AppCommandOptionType.role),
-    AppCommandChannel: channel_transformer(AppCommandChannel, raw=True),
-    AppCommandThread: channel_transformer(AppCommandThread, raw=True),
-    Thread: channel_transformer(Thread),
-    StageChannel: channel_transformer(StageChannel),
-    VoiceChannel: channel_transformer(VoiceChannel),
-    TextChannel: channel_transformer(TextChannel),
-    CategoryChannel: channel_transformer(CategoryChannel),
-    Attachment: passthrough_transformer(AppCommandOptionType.attachment),
+BUILT_IN_TRANSFORMERS: Dict[Any, Transformer] = {
+    str: IdentityTransformer(AppCommandOptionType.string),
+    int: IdentityTransformer(AppCommandOptionType.integer),
+    float: IdentityTransformer(AppCommandOptionType.number),
+    bool: IdentityTransformer(AppCommandOptionType.boolean),
+    User: IdentityTransformer(AppCommandOptionType.user),
+    Member: MemberTransformer(),
+    Role: IdentityTransformer(AppCommandOptionType.role),
+    AppCommandChannel: RawChannelTransformer(AppCommandChannel),
+    AppCommandThread: RawChannelTransformer(AppCommandThread),
+    GuildChannel: BaseChannelTransformer(GuildChannel),
+    Thread: BaseChannelTransformer(Thread),
+    StageChannel: BaseChannelTransformer(StageChannel),
+    VoiceChannel: BaseChannelTransformer(VoiceChannel),
+    TextChannel: BaseChannelTransformer(TextChannel),
+    CategoryChannel: BaseChannelTransformer(CategoryChannel),
+    Attachment: IdentityTransformer(AppCommandOptionType.attachment),
 }
 
 ALLOWED_DEFAULTS: Dict[AppCommandOptionType, Tuple[Type[Any], ...]] = {
@@ -612,45 +732,58 @@ def get_supported_annotation(
     annotation: Any,
     *,
     _none: type = NoneType,
-    _mapping: Dict[Any, Type[Transformer]] = BUILT_IN_TRANSFORMERS,
-) -> Tuple[Any, Any]:
+    _mapping: Dict[Any, Transformer] = BUILT_IN_TRANSFORMERS,
+) -> Tuple[Any, Any, bool]:
     """Returns an appropriate, yet supported, annotation along with an optional default value.
 
+    The third boolean element of the tuple indicates if default values should be validated.
+
     This differs from the built in mapping by supporting a few more things.
     Likewise, this returns a "transformed" annotation that is ready to use with CommandParameter.transform.
     """
 
     try:
-        return (_mapping[annotation], MISSING)
+        return (_mapping[annotation], MISSING, True)
     except KeyError:
         pass
 
-    if hasattr(annotation, '__discord_app_commands_transform__'):
-        return (annotation.metadata, MISSING)
+    if isinstance(annotation, Transformer):
+        return (annotation, MISSING, False)
 
     if hasattr(annotation, '__metadata__'):
         return get_supported_annotation(annotation.__metadata__[0])
 
     if inspect.isclass(annotation):
         if issubclass(annotation, Transformer):
-            return (annotation, MISSING)
+            return (annotation(), MISSING, False)
         if issubclass(annotation, (Enum, InternalEnum)):
             if all(isinstance(v.value, (str, int, float)) for v in annotation):
-                return (_make_enum_transformer(annotation), MISSING)
+                return (EnumValueTransformer(annotation), MISSING, False)
             else:
-                return (_make_complex_enum_transformer(annotation), MISSING)
+                return (EnumNameTransformer(annotation), MISSING, False)
         if annotation is Choice:
             raise TypeError(f'Choice requires a type argument of int, str, or float')
 
+        # Check if a transform @classmethod is given to the class
+        # These flatten into simple "inline" transformers with implicit strings
+        transform_classmethod = annotation.__dict__.get('transform', None)
+        if isinstance(transform_classmethod, classmethod):
+            params = inspect.signature(transform_classmethod.__func__).parameters
+            if len(params) != 3:
+                raise TypeError(f'Inline transformer with transform classmethod requires 3 parameters')
+            if not inspect.iscoroutinefunction(transform_classmethod.__func__):
+                raise TypeError(f'Inline transformer with transform classmethod must be a coroutine')
+            return (InlineTransformer(annotation), MISSING, False)
+
     # Check if there's an origin
     origin = getattr(annotation, '__origin__', None)
     if origin is Literal:
         args = annotation.__args__  # type: ignore
-        return (_make_literal_transformer(args), MISSING)
+        return (LiteralTransformer(args), MISSING, True)
 
     if origin is Choice:
         arg = annotation.__args__[0]  # type: ignore
-        return (_make_choice_transformer(arg), MISSING)
+        return (ChoiceTransformer(arg), MISSING, True)
 
     if origin is not Union:
         # Only Union/Optional is supported right now so bail early
@@ -661,10 +794,10 @@ def get_supported_annotation(
     if args[-1] is _none:
         if len(args) == 2:
             underlying = args[0]
-            inner, _ = get_supported_annotation(underlying)
+            inner, _, validate_default = get_supported_annotation(underlying)
             if inner is None:
                 raise TypeError(f'unsupported inner optional type {underlying!r}')
-            return (inner, None)
+            return (inner, None, validate_default)
         else:
             args = args[:-1]
             default = None
@@ -672,7 +805,7 @@ def get_supported_annotation(
     # Check for channel union types
     if any(arg in CHANNEL_TO_TYPES for arg in args):
         # If any channel type is given, then *all* must be channel types
-        return (channel_transformer(*args, raw=None), default)
+        return (UnionChannelTransformer(*args), default, True)
 
     # The only valid transformations here are:
     # [Member, User] => user
@@ -682,9 +815,9 @@ def get_supported_annotation(
     if not all(arg in supported_types for arg in args):
         raise TypeError(f'unsupported types given inside {annotation!r}')
     if args == (User, Member) or args == (Member, User):
-        return (passthrough_transformer(AppCommandOptionType.user), default)
+        return (IdentityTransformer(AppCommandOptionType.user), default, True)
 
-    return (passthrough_transformer(AppCommandOptionType.mentionable), default)
+    return (IdentityTransformer(AppCommandOptionType.mentionable), default, True)
 
 
 def annotation_to_parameter(annotation: Any, parameter: inspect.Parameter) -> CommandParameter:
@@ -695,8 +828,8 @@ def annotation_to_parameter(annotation: Any, parameter: inspect.Parameter) -> Co
     of a command parameter.
     """
 
-    (inner, default) = get_supported_annotation(annotation)
-    type = inner.type()
+    (inner, default, validate_default) = get_supported_annotation(annotation)
+    type = inner.type
 
     if default is MISSING or default is None:
         param_default = parameter.default
@@ -704,12 +837,10 @@ def annotation_to_parameter(annotation: Any, parameter: inspect.Parameter) -> Co
             default = param_default
 
     # Verify validity of the default parameter
-    if default is not MISSING:
-        enum_type = getattr(inner, '__discord_app_commands_transformer_enum__', None)
-        if default.__class__ is not enum_type:
-            valid_types: Tuple[Any, ...] = ALLOWED_DEFAULTS.get(type, (NoneType,))
-            if not isinstance(default, valid_types):
-                raise TypeError(f'invalid default parameter type given ({default.__class__}), expected {valid_types}')
+    if default is not MISSING and validate_default:
+        valid_types: Tuple[Any, ...] = ALLOWED_DEFAULTS.get(type, (NoneType,))
+        if not isinstance(default, valid_types):
+            raise TypeError(f'invalid default parameter type given ({default.__class__}), expected {valid_types}')
 
     result = CommandParameter(
         type=type,
@@ -719,28 +850,25 @@ def annotation_to_parameter(annotation: Any, parameter: inspect.Parameter) -> Co
         name=parameter.name,
     )
 
-    try:
-        choices = inner.__discord_app_commands_transformer_choices__
-    except AttributeError:
-        pass
-    else:
+    choices = inner.choices
+    if choices is not None:
         result.choices = choices
 
     # These methods should be duck typed
-    if type in (AppCommandOptionType.number, AppCommandOptionType.integer):
-        result.min_value = inner.min_value()
-        result.max_value = inner.max_value()
+    if type in (AppCommandOptionType.number, AppCommandOptionType.string, AppCommandOptionType.integer):
+        result.min_value = inner.min_value
+        result.max_value = inner.max_value
 
     if type is AppCommandOptionType.channel:
-        result.channel_types = inner.channel_types()
+        result.channel_types = inner.channel_types
 
     if parameter.kind in (parameter.POSITIONAL_ONLY, parameter.VAR_KEYWORD, parameter.VAR_POSITIONAL):
         raise TypeError(f'unsupported parameter kind in callback: {parameter.kind!s}')
 
-    autocomplete_func = getattr(inner.autocomplete, '__func__', inner.autocomplete)
-    if autocomplete_func is not Transformer.autocomplete.__func__:
+    # Check if the method is overridden
+    if inner.autocomplete.__func__ is not Transformer.autocomplete:
         from .commands import _validate_auto_complete_callback
 
-        result.autocomplete = _validate_auto_complete_callback(inner.autocomplete, skip_binding=True)
+        result.autocomplete = _validate_auto_complete_callback(inner.autocomplete)
 
     return result
