@@ -291,6 +291,7 @@ class DiscordWebSocket:
         _max_heartbeat_timeout: float
 
     # fmt: off
+    DEFAULT_GATEWAY    = 'wss://gateway.discord.gg/'
     DISPATCH           = 0
     HEARTBEAT          = 1
     IDENTIFY           = 2
@@ -350,13 +351,24 @@ class DiscordWebSocket:
         session: Optional[str] = None,
         sequence: Optional[int] = None,
         resume: bool = False,
+        encoding: str = 'json',
+        zlib: bool = True,
     ) -> Self:
         """Creates a main websocket for Discord from a :class:`Client`.
 
         This is for internal use only.
         """
-        gateway = gateway or await client.http.get_gateway()
-        socket = await client.http.ws_connect(gateway)
+        # Circular import
+        from .http import INTERNAL_API_VERSION
+
+        gateway = gateway or cls.DEFAULT_GATEWAY
+
+        if zlib:
+            url = f'{gateway}?v={INTERNAL_API_VERSION}&encoding={encoding}&compress=zlib-stream'
+        else:
+            url = f'{gateway}?v={INTERNAL_API_VERSION}&encoding={encoding}'
+
+        socket = await client.http.ws_connect(url)
         ws = cls(socket, loop=client.loop)
 
         # dynamically add attributes needed
@@ -429,11 +441,9 @@ class DiscordWebSocket:
             'd': {
                 'token': self.token,
                 'properties': {
-                    '$os': sys.platform,
-                    '$browser': 'discord.py',
-                    '$device': 'discord.py',
-                    '$referrer': '',
-                    '$referring_domain': '',
+                    'os': sys.platform,
+                    'browser': 'discord.py',
+                    'device': 'discord.py',
                 },
                 'compress': True,
                 'large_threshold': 250,
@@ -535,6 +545,7 @@ class DiscordWebSocket:
 
                 self.sequence = None
                 self.session_id = None
+                self.gateway = self.DEFAULT_GATEWAY
                 _log.info('Shard ID %s session has been invalidated.', self.shard_id)
                 await self.close(code=1000)
                 raise ReconnectWebSocket(self.shard_id, resume=False)
@@ -543,26 +554,15 @@ class DiscordWebSocket:
             return
 
         if event == 'READY':
-            self._trace = trace = data.get('_trace', [])
             self.sequence = msg['s']
             self.session_id = data['session_id']
-            _log.info(
-                'Shard ID %s has connected to Gateway: %s (Session ID: %s).',
-                self.shard_id,
-                ', '.join(trace),
-                self.session_id,
-            )
+            self.gateway = data['resume_gateway_url']
+            _log.info('Shard ID %s has connected to Gateway (Session ID: %s).', self.shard_id, self.session_id)
 
         elif event == 'RESUMED':
-            self._trace = trace = data.get('_trace', [])
             # pass back the shard ID to the resumed handler
             data['__shard_id__'] = self.shard_id
-            _log.info(
-                'Shard ID %s has successfully RESUMED session %s under trace %s.',
-                self.shard_id,
-                self.session_id,
-                ', '.join(trace),
-            )
+            _log.info('Shard ID %s has successfully RESUMED session %s.', self.shard_id, self.session_id)
 
         try:
             func = self._discord_parsers[event]
@@ -826,7 +826,7 @@ class DiscordVoiceWebSocket:
         self._close_code: Optional[int] = None
         self.secret_key: Optional[str] = None
         if hook:
-            self._hook = hook  # type: ignore # type-checker doesn't like overriding methods
+            self._hook = hook
 
     async def _hook(self, *args: Any) -> None:
         pass
